@@ -1,8 +1,8 @@
 /************************************************************************
 
-    SdlTerm v1.00 - Terminal emulation in an SDL framebuffer
+    SdlTerm v1.01 - Terminal emulation in an SDL framebuffer
 
-    Copyright (C) 1996, 2001  John Elliott <jce@seasip.demon.co.uk>
+    Copyright (C) 1996, 2001, 2006  John Elliott <jce@seasip.demon.co.uk>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,14 +31,41 @@ static Uint8 fontDef[] =
 
 SdlTerm::SdlTerm(SDL_Surface *s, SDL_Rect *bounds) : SdlContext(s, bounds)
 {
+	int n, m;
+	SXLOG("SdlTerm::SdlTerm()");
+	for (n = 0; n < SDLK_LAST; n++)
+	{
+		for (m = 0; m < 8; m++)
+		{
+		        m_keyMap[m][n] = new std::string;
+		}	
+	}
 	m_font    = NULL;
 	reset();
 	resetInput();
+	SXLOG("SdlTerm::SdlTerm() done");
+
 }
 
 SdlTerm::~SdlTerm()
 {
-	if (m_font) delete m_font;
+	int n, m;
+
+	SXLOG("SdlTerm::~SdlTerm()");
+	
+	if (m_font) 
+	{
+		delete m_font;
+		m_font = NULL;
+	}
+	for (n = 0; n < SDLK_LAST; n++)
+	{
+		for (m = 0; m < 8; m++)
+		{
+		        delete m_keyMap[m][n];
+		}	
+	}
+	SXLOG("SdlTerm::~SdlTerm() done");
 }
 
 
@@ -128,6 +155,42 @@ void SdlTerm::setFont(Uint8 *psf)
 }
 
 
+void SdlTerm::beginMergeUpdate(void)
+{
+	m_x0 = m_bounds.x + m_bounds.w;
+	m_x1 = m_bounds.x;
+	m_y0 = m_bounds.y + m_bounds.h;
+	m_y1 = m_bounds.y;
+	m_oldManualUpdate = m_manualUpdate;
+	m_manualUpdate = true;
+}
+
+void SdlTerm::condUpdate(SDL_Rect *rc)
+{
+	condUpdate(rc->x, rc->y, rc->w, rc->h);
+}
+
+void SdlTerm::condUpdate(int x, int y, int w, int h)
+{
+	if (m_manualUpdate)
+	{
+		if (m_x0 > x)		m_x0 = x;
+		if (m_x1 < (x + w)) 	m_x1 = x + w;
+		if (m_y0 > y)		m_y0 = y;
+		if (m_y1 < (y + h)) 	m_y1 = y + h;	
+	}
+	else	SDL_UpdateRect(m_surface, x, y, w, h);
+}
+
+
+void SdlTerm::endMergeUpdate(void)
+{
+	if (m_x0 < m_x1 && m_y0 < m_y1 && !m_oldManualUpdate)
+	{
+		SDL_UpdateRect(m_surface, m_x0, m_y0, m_x1 - m_x0, m_y1 - m_y0);
+	}
+	m_manualUpdate = m_oldManualUpdate;
+}
 
 SdlTerm& SdlTerm::operator << (unsigned char c)
 {
@@ -137,11 +200,13 @@ SdlTerm& SdlTerm::operator << (unsigned char c)
 
 SdlTerm& SdlTerm::operator << (const char *s)
 {
+	beginMergeUpdate();
 	while (*s)
 	{
 		operator <<((unsigned char)(*s));
 		++s;
 	}
+	endMergeUpdate();
 	return *this;
 }
 
@@ -308,7 +373,7 @@ void SdlTerm::deleteChar(void)
         sr.x += m_charw;
         blit(sr, dr);
 	dr.w += m_charw;
-        if (!m_manualUpdate) SDL_UpdateRects(m_surface, 1, &dr);
+	condUpdate(&dr);
 }
 
 
@@ -620,57 +685,6 @@ void SdlTerm::blit(SDL_Rect &sr, SDL_Rect &dr)
 }
 
 
-#if 0
-void vga_setmode(Uint8 mode)	/* Switch between PCW mode and native mode */
-{
-	if (mode == ScrType) return;
-
-	switch(mode)
-	{
-		case 0: /* PCW */
-		sel_language(0);
-		shadow_invalid = 1;
-		ScrType = 0;
-		break;
-
-		case 1:	/* SVGA */
-		GrClearScreen(dblk);
-		vga_fg  = dwhi;
-		vga_bg  = dblk;
-		rv      = 0;
-		ul      = 0;
-		bold 	= 0;
-		italic	= 0;
-		grey	= 0;
-		onesc   = 0;
-		onplus  = 0;
-		wrap    = 1;
-		wtop    = 0;
-		wleft   = 0;
-		pgmode	= 0;
-		statln  = 1;
-		wbot    = SCRBOT;
-		wright  = 100;
-		vga_curon   = 1;
-		vga_curvis  = 0;
-		autox   = 1;
-		set_window();
-		sel_language(0);
-#ifdef USE_SDL
-	        reload_palette();
-#endif
-		UpdateArea(0, 0, 0, 0);	/* Update whole screen */
-		ScrType = 1;
-		break;
-
-		default:
-		joyce_dprintf("Unknown screen mode %d\n", mode);
-		break;
-	}
-}
-
-
-#endif
 void SdlTerm::enableStatus(bool b)
 {
 	int wb = m_windowC.y + m_windowC.h;
@@ -741,7 +755,7 @@ void SdlTerm::scrollUp(void)
 	blit(sr, dr);
 
 	dr.h += m_charh;	
-	if (!m_manualUpdate) SDL_UpdateRects(m_surface, 1, &dr);
+	condUpdate(&dr);
 	clearCurLine();
 }
 
@@ -770,7 +784,7 @@ void SdlTerm::scrollLineRangeDown(int first, int last)
 					24, 0, 0, 0, 0);
 
 	sr.h += m_charh;
-        if (!m_manualUpdate) SDL_UpdateRects(m_surface, 1, &sr);
+	condUpdate(&sr);
 }
 
 
@@ -797,7 +811,7 @@ void SdlTerm::scrollLineRangeUp(int first, int last)
         sr.y += m_charh;
 	blit(sr, dr);
 	dr.h += m_charh;
-        if (!m_manualUpdate) SDL_UpdateRects(m_surface, 1, &dr);
+	condUpdate(&dr);
 }
 
 
@@ -815,7 +829,7 @@ void SdlTerm::scrollDown(void)
         dr.y += m_charh;
 	blit(sr, dr);
 	sr.h += m_charh;
-        if (!m_manualUpdate) SDL_UpdateRects(m_surface, 1, &sr);
+	condUpdate(&sr);
         clearCurLine();
 }
 
@@ -903,7 +917,7 @@ void SdlTerm::rawGlyph(int x, int y, Uint8 *bitmap, int rast_op)
 		bits += m_surface->pitch;
 	}
 	unlock();
-	if (!m_manualUpdate) SDL_UpdateRect(m_surface, x, y, m_charw, m_charh);
+	condUpdate(x, y, m_charw, m_charh);
 }
 
 #if 0
@@ -967,7 +981,7 @@ void SdlTerm::drawCursor()
 		bits += m_surface->pitch;
 	}
 	unlock();
-	if (!m_manualUpdate) SDL_UpdateRect(m_surface, x, y + y0, x + m_charw, y1 - y0);
+	condUpdate(x, y + y0, x + m_charw, y1 - y0);
 }
 
 
