@@ -1,6 +1,6 @@
 /************************************************************************
 
-    JOYCE v1.90 - Amstrad PCW emulator
+    JOYCE v2.1.0 - Amstrad PCW emulator
 
     Copyright (C) 1996, 2001  John Elliott <jce@seasip.demon.co.uk>
 
@@ -29,7 +29,7 @@
 #include "UiMenu.hxx"
 #include "UiTextInput.hxx"
 #include "UiScrollingMenu.hxx"
-#include "JoyceFileList.hxx"
+#include "PcwFileList.hxx"
 
 #define DAISY_MAX_H 3300	// Longer than an A3 page
 
@@ -82,7 +82,7 @@ static unsigned char gl_wheel[128] =
 };
 #undef XX
 
-JoyceDaisy::JoyceDaisy(JoyceSystem *sys) : JoycePrinter("daisy"), m_sys(sys)
+JoyceDaisy::JoyceDaisy(JoyceSystem *sys) : PcwPrinter("daisy"), m_sys(sys)
 {
 	XLOG("JoyceDaisy::JoyceDaisy()");
 	reset();
@@ -90,14 +90,14 @@ JoyceDaisy::JoyceDaisy(JoyceSystem *sys) : JoycePrinter("daisy"), m_sys(sys)
 	m_resultw = m_resultr = 0;
 	m_daisyEnabled = true;
 	m_intTime = 0;
+	m_format = PS;
 
         m_fp = NULL;
         m_w =  999;     // A4 width
         m_h = 2245;     // A4 height
         m_y = 192;      // Start 1 inch down
 	m_x = 0;
-        m_psdir = findJoyceDir(FT_PS, false);
-	m_filename = "";
+        m_psdir = findPcwDir(FT_PS, false);
 	m_drawing_page = false;
 	m_sequence = 0;
 	// Initialise printwheel data
@@ -113,7 +113,8 @@ JoyceDaisy::JoyceDaisy(JoyceSystem *sys) : JoycePrinter("daisy"), m_sys(sys)
 
 JoyceDaisy::~JoyceDaisy()
 {
-        endPagePS();
+	XLOG("JoyceDaisy::~JoyceDaisy()");
+        endPage();
         if (m_fp) closePS();
 }
 
@@ -125,6 +126,30 @@ void JoyceDaisy::reset()
 }
 
 
+byte JoyceDaisy::parInp()
+{
+        unsigned busyline;
+        ll_error_t e;
+	byte r = 0;
+
+        if (!isEnabled()) return 0x40;
+
+        if (!m_lldev) open();
+        if (!m_lldev) return 0x40;
+
+        e = ll_rctl_poll(m_lldev, &busyline);
+        showError(e);
+        if (e != LL_E_OK) return 0x40;
+        if (busyline & LL_CTL_ACK)     r |= 0x80;
+        if (busyline & LL_CTL_BUSY)    r |= 0x40;
+        if (busyline & LL_CTL_ERROR)   r |= 0x20;
+        if (busyline & LL_CTL_ISELECT) r |= 0x10;
+        if (busyline & LL_CTL_NOPAPER) r |= 0x08;
+        return r;
+}
+	
+
+
 
 void JoyceDaisy::out(word port, byte value)
 {
@@ -172,8 +197,8 @@ void JoyceDaisy::out(word port, byte value)
 			if (m_cmd[0] & 0x3F)
 			{
 				char c = m_wheel[m_cmd[1] & 0x7F];
-			        if (m_drawing_page || startPagePS())
-					charPS(c, m_x, m_y);	
+			        if (m_drawing_page || startPage())
+					putChar(c, m_x, m_y);	
 			}
 			m_intTime = 1000;
 		}
@@ -183,7 +208,7 @@ void JoyceDaisy::out(word port, byte value)
 					// parameters. We ignore these for now.
 					break;
 			// PAR status
-			case 0x02:	addResult(isBusy() ? 0x40 : 0);
+			case 0x02:	addResult(parInp());
 					break;
 			case 0x04:	writeChar(m_cmd[1]);	
 					break;
@@ -201,7 +226,7 @@ void JoyceDaisy::out(word port, byte value)
 					m_intEnabled = false;
 					break;
 // Acknowledge interrupt
-			case 0x09:      m_sys->m_asic.daisyIsr(false);
+			case 0x09:      m_sys->m_asic->daisyIsr(false);
 					m_interrupting = false;
 					break;	
 			case 0x0A:	// dummy
@@ -302,7 +327,8 @@ UiEvent JoyceDaisy::settings(UiDrawer *d)
                         case SDLK_d: e = daisySettings(d);
                                      if (e == UIE_QUIT) return e;
                                      break;
-                        case SDLK_p: e = JoycePrinter::settings(d);
+                        //case SDLK_p: e = PcwPrinter::settings(d);
+                        case SDLK_p: e = PcwPrinter::settings(d);
                                      if (e == UIE_QUIT) return e;
                                      break;
                         default:     break;
@@ -381,8 +407,8 @@ bool JoyceDaisy::parsePrintWheel(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
 				ch  = atoi(c);
 				if (idx >= 0 && idx < sizeof(m_wheel)) m_wheel[idx] = ch;
 			}
-			if (i) free(i);
-			if (c) free(c);
+			if (i) xmlFree(i);
+			if (c) xmlFree(c);
 		}
 		return true;
         }
@@ -392,29 +418,40 @@ bool JoyceDaisy::parsePrintWheel(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
 bool JoyceDaisy::parseNode(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
 {
 	char *t;
-	if (!JoycePrinter::parseNode(doc, ns, cur)) return false;
+	if (!PcwPrinter::parseNode(doc, ns, cur)) return false;
      
 	if (!parsePrintWheel(doc, ns, cur)) return false;
   
 	parseFilename(doc, ns, cur, "psoutputdir", m_psdir);
 
         t = (char *)xmlGetProp(cur, (xmlChar *)"width");
-        if (t) { m_w = atoi(t); free(t); }
+        if (t) { m_w = atoi(t); xmlFree(t); }
         t = (char *)xmlGetProp(cur, (xmlChar *)"height");
-        if (t) { m_h = atoi(t); free(t); }
+        if (t) { m_h = atoi(t); xmlFree(t); }
         t = (char *)xmlGetProp(cur, (xmlChar *)"fontsize");
-        if (t) { m_ptsize = atoi(t); free(t); }
+        if (t) { m_ptsize = atoi(t); xmlFree(t); }
         t = (char *)xmlGetProp(cur, (xmlChar *)"fontname");
-        if (t) { m_fontname = t; free(t); }
-       
+        if (t) { m_fontname = t; xmlFree(t); }
+      
+        t = (char *)xmlGetProp(cur, (xmlChar *)"format");
+        if (t)
+        {
+                if (!strcmp(t, "PS"))  m_format = PS;
+#ifdef HAVE_WINDOWS_H
+                if (!strcmp(t, "GDI")) m_format = GDI;
+#endif
+                xmlFree(t);
+        }
+	
+ 
 	char *s = (char *)xmlGetProp(cur, (xmlChar *)"daisyenabled");
 
         if (!s) return true;    // Default setting used
 
-        if (!strcmp(s, "1")) { m_daisyEnabled = true;  return true; }
-        if (!strcmp(s, "0")) { m_daisyEnabled = false; return true; }
-
+        if (!strcmp(s, "1")) { m_daisyEnabled = true;  xmlFree(s); return true; }
+        if (!strcmp(s, "0")) { m_daisyEnabled = false; xmlFree(s); return true; }
         joyce_dprintf("'enabled=%s' option not recognised.\n", s);
+	xmlFree(s);
         return false;
 }
 
@@ -432,8 +469,16 @@ bool JoyceDaisy::storeNode(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
 	xmlSetProp(cur, (xmlChar *)"fontname", (xmlChar *)m_fontname.c_str());
         sprintf(buf, "%d", m_ptsize);
         xmlSetProp(cur, (xmlChar *)"fontsize", (xmlChar *)buf);
+        switch(m_format)
+        {
+                case PS:   xmlSetProp(cur, (xmlChar *)"format", (xmlChar *)"PS");
+#ifdef HAVE_WINDOWS_H
+                case GDI:  xmlSetProp(cur, (xmlChar *)"format", (xmlChar *)"GDI");
+#endif
+        }
+
 	storePrintWheel(doc, ns, cur);	
-        return JoycePrinter::storeNode(doc, ns, cur);
+        return PcwPrinter::storeNode(doc, ns, cur);
 }
 
 
@@ -445,6 +490,7 @@ UiEvent JoyceDaisy::daisySettings(UiDrawer *d)
 	bool bEnabled;
 	UiEvent uie;
 	char cust[80];
+	PrinterFormat format = m_format;
 
 	// Should we add settings for PostScript font & point size, or
 	// leave them (along with printwheel) for advanced users who 
@@ -455,6 +501,8 @@ UiEvent JoyceDaisy::daisySettings(UiDrawer *d)
 	while (1)
 	{
 		UiMenu uim(d);
+		string prevfn;
+		string prevdr;
 
 		a3  = (m_w == 1403  && m_h == 3196);
 		a4l = (m_w == 1403  && m_h == 1598);
@@ -490,10 +538,18 @@ UiEvent JoyceDaisy::daisySettings(UiDrawer *d)
 			string ostat = "  Output is to ";
 			if (m_psdir[0] == '|') ostat += "pipe: " + displayName(m_psdir.substr(1), 40);
 			else		       ostat += "directory: " + displayName(m_psdir, 40);
+#ifdef HAVE_WINDOWS_H
 			uim.add(new UiSeparator(d));
-			uim.add(new UiLabel  (ostat, d)); 
-			uim.add(new UiCommand(SDLK_r,  "  Output to diRectory  ", d, UIG_SUBMENU));
-			uim.add(new UiCommand(SDLK_u,  "  Output to UNIX command... ",d, UIG_SUBMENU));
+			uim.add(new UiSetting(SDLK_p, format == PS,  "  Output in PostScript format  ", d));
+			uim.add(new UiSetting(SDLK_g, format == GDI, "  Output using Windows' GDI  ", d));
+#endif
+			if (m_format == PS) 
+			{
+				uim.add(new UiSeparator(d));
+				uim.add(new UiLabel  (ostat, d)); 
+				uim.add(new UiCommand(SDLK_r,  "  Output to diRectory  ", d, UIG_SUBMENU));
+				uim.add(new UiCommand(SDLK_u,  "  Output to UNIX command... ",d, UIG_SUBMENU));
+			}
 		}
 
 		d->centreContainer(uim);
@@ -517,24 +573,32 @@ UiEvent JoyceDaisy::daisySettings(UiDrawer *d)
                         case SDLK_0:    uie = getCustomPaper(d);
                                         if (uie == UIE_QUIT) return uie;
                                         break;
-
+			case SDLK_p: format = PS; break;
+#ifdef HAVE_WINDOWS_H
+			case SDLK_g: format = GDI; break;
+#endif
 			case SDLK_d: m_daisyEnabled = false; break;
 			case SDLK_c: m_daisyEnabled = true;  break;
                         case SDLK_r:    m_y = DAISY_MAX_H + 1;
                                         checkEndForm();
-                                        m_psdir = findJoyceDir(FT_PS, false);
+                                        m_psdir = findPcwDir(FT_PS, false);
                                         break;
-                        case SDLK_u:    m_y = DAISY_MAX_H + 1;
+                        case SDLK_u:    prevdr = m_lldriver;
+					prevfn = m_llfilename;
+					m_y = DAISY_MAX_H + 1;
                                         checkEndForm();
                                         uie = getPipe(d);
                                         if (uie == UIE_QUIT) return uie;
-                                        m_psdir = m_filename;
+                                        m_psdir = "|";
+					m_psdir += m_llfilename;
+					m_llfilename = prevfn;
+					m_lldriver = prevdr;
                                         break;
 
 			case SDLK_ESCAPE: close();  return UIE_OK;
 			default: break;
 		}
-
+		m_format = format;
 	}
 	return UIE_OK;
 }
@@ -542,7 +606,7 @@ UiEvent JoyceDaisy::daisySettings(UiDrawer *d)
 
 void JoyceDaisy::interrupt(void)
 {
-	m_sys->m_asic.daisyIsr(true);
+	m_sys->m_asic->daisyIsr(true);
 	m_interrupting = true;
 }
 
@@ -561,6 +625,28 @@ void JoyceDaisy::tick()
 }
 
 
+bool JoyceDaisy::startPage(void)
+{
+        switch(m_format)
+        {
+                case PS:  return startPagePS();
+#ifdef HAVE_WINDOWS_H
+                case GDI:
+                          if (m_drawing_page) return true;
+// [JOYCE 2.1.1] Correct the scaling 
+			  m_gdiPrint.m_fontSize = m_ptsize * 24;
+			  if (m_fontname == "Courier") 
+				m_gdiPrint.m_fontFace = "Courier New";
+			  else  m_gdiPrint.m_fontFace = m_fontname;
+
+                          m_drawing_page = m_gdiPrint.startPage();
+                          return m_drawing_page;
+#endif
+        }
+        return false;
+
+}
+
 ////////////////////////////////////////////////////////////////////////////
 //
 // Output page as a Postscript file
@@ -581,13 +667,13 @@ bool JoyceDaisy::startPagePS(void)
 	
 	while (!m_fp)	
 	{
-		if (m_psdir == "") m_psdir = ".";	
 		if (m_psdir[0] == '|')
 		{
 			m_fp = popen(m_psdir.c_str() + 1, "w");
 		}
 		else
 		{	
+			if (m_psdir == "") m_psdir = ".";	
 			if (m_psdir[m_psdir.length() - 1] == '/')
 				m_psdir = m_psdir.substr(0, m_psdir.length() -1);
 			checkExistsDir(m_psdir);
@@ -644,6 +730,35 @@ bool JoyceDaisy::endPagePS(void)
 	return true;
 }
 
+bool JoyceDaisy::endPage(void)
+{
+        switch(m_format)
+        {
+                case PS:  return endPagePS();
+#ifdef HAVE_WINDOWS_H
+                case GDI: m_drawing_page = false;
+                          return m_gdiPrint.endPage();
+#endif
+        }
+        return false;
+}
+
+
+void JoyceDaisy::putChar(char c, float xf, float yf)
+{
+        switch(m_format)
+        {
+                case PS:  charPS(c, xf, yf);
+#ifdef HAVE_WINDOWS_H
+// [JOYCE v2.1.1: Convert from 120/192dpi to 360dpi]
+                case GDI: m_gdiPrint.putChar(xf * 3.0, yf * 1.875, c);
+#endif
+        }
+}
+
+
+
+
 void JoyceDaisy::charPS(char c, float xf, float yf)
 {
 	if (!m_fp) return;
@@ -668,7 +783,7 @@ void JoyceDaisy::checkEndForm(void)
 {
         if (m_y >= DAISY_MAX_H && m_drawing_page)
         {
-                endPagePS();
+                endPage();
                 m_y = 360;      /* Feed a new sheet; and we can't print
                                  * on the top 6 lines */
         }
