@@ -29,7 +29,7 @@
 #include "UiMenu.hxx"
 #include "UiTextInput.hxx"
 #include "UiScrollingMenu.hxx"
-#include "JoyceFileList.hxx"
+#include "PcwFileList.hxx"
 //#include "JoyceDirChooser.hxx"
 
 /* Define png_jmpbuf() in case we are using a pre-1.0.6 version of libpng */
@@ -43,7 +43,7 @@
 #define K_ADJUST 9
 #define L_ADJUST 11
 
-JoyceMatrix::JoyceMatrix() : JoycePrinter("matrix")
+JoyceMatrix::JoyceMatrix() : PcwPrinter("matrix")
 {
 	XLOG("JoyceMatrix::JoyceMatrix()");
 	m_noRam = false;
@@ -55,8 +55,8 @@ JoyceMatrix::JoyceMatrix() : JoycePrinter("matrix")
 	m_w = 2997;	// A4 width
 	m_h = 4208;	// A4 height
 	m_y = 360;	// Start 1 inch down
-	m_pngdir = findJoyceDir(FT_PNGS, false);
-	m_psdir = findJoyceDir(FT_PS, false);
+	m_pngdir = findPcwDir(FT_PNGS, false);
+	m_psdir = findPcwDir(FT_PS, false);
 	m_bailIn = true;
 	for (int r = 0; r < MAX_H; r++)
 	{
@@ -70,6 +70,7 @@ JoyceMatrix::JoyceMatrix() : JoycePrinter("matrix")
 
 JoyceMatrix::~JoyceMatrix()
 {
+	XLOG("JoyceMatrix::~JoyceMatrix()");
 	endPage();
 	if (m_fp) closePS();
 	for (int r = 0; r < MAX_H; r++)
@@ -139,15 +140,18 @@ bool JoyceMatrix::parseNode(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
 	parseFilename(doc, ns, cur, "psoutputdir", m_psdir);
 
         t = (char *)xmlGetProp(cur, (xmlChar *)"width");
-        if (t) { m_w = atoi(t); free(t); }
+        if (t) { m_w = atoi(t); xmlFree(t); }
         t = (char *)xmlGetProp(cur, (xmlChar *)"height");
-        if (t) { m_h = atoi(t); free(t); }
+        if (t) { m_h = atoi(t); xmlFree(t); }
         t = (char *)xmlGetProp(cur, (xmlChar *)"format");
         if (t) 
 	{ 
 		if (!strcmp(t, "PS"))  m_format = PS;
 		if (!strcmp(t, "PNG")) m_format = PNG;
-		free(t);
+#ifdef HAVE_WINDOWS_H
+		if (!strcmp(t, "GDI")) m_format = GDI;
+#endif
+		xmlFree(t);
 	}
 
 	return true;
@@ -180,6 +184,9 @@ bool JoyceMatrix::storeNode(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
 	{
 		case PNG:  xmlSetProp(cur, (xmlChar *)"format", (xmlChar *)"PNG"); break;
 		case PS:   xmlSetProp(cur, (xmlChar *)"format", (xmlChar *)"PS");  break;
+#ifdef HAVE_WINDOWS_H
+		case GDI:  xmlSetProp(cur, (xmlChar *)"format", (xmlChar *)"GDI");  break;
+#endif
 	}
 	return true;
 }
@@ -326,7 +333,7 @@ void JoyceMatrix::dots(void)
 static void matrix_png_error(png_structp png_ptr, png_const_charp error_msg)
 {
 	JoyceMatrix *m = (JoyceMatrix *)(png_ptr->error_ptr);	
-	UiDrawer *d = &(gl_sys->m_termMenu);
+	UiDrawer *d = gl_sys->m_termMenu;
         UiMenu uim(d);
 	string sl1,sl2;
        
@@ -519,8 +526,9 @@ UiEvent JoyceMatrix::settings(UiDrawer *d)
 	bool a4, a5, a5l, cont11, cont12, custom;
         x = y = 0;
         sel = -1;
-	MatrixFormat format = m_format;
+	PrinterFormat format = m_format;
 	string strdir;
+	string oldfn, olddr;
 
         while (1)
         {
@@ -562,14 +570,22 @@ UiEvent JoyceMatrix::settings(UiDrawer *d)
                 	uim.add(new UiSeparator(d));
                 	uim.add(new UiSetting(SDLK_n, (format == PNG), "  Output PNG files",   d));
                 	uim.add(new UiSetting(SDLK_o, (format == PS),  "  Output PostScript",  d));
-				
-                        string ostat = "  Output is to ";
-                        if (strdir[0] == '|') ostat += "pipe: " + displayName(strdir.substr(1), 40) + "  ";
-                        else                  ostat += "directory: " + displayName(strdir, 40) + "  "; 
-                        uim.add(new UiSeparator(d));
-                        uim.add(new UiLabel  (ostat, d));
-                        uim.add(new UiCommand(SDLK_r,  "  Output to diRectory  ", d));
-			uim.add(new UiCommand(SDLK_u,  "  Output to UNIX command... ",d, UIG_SUBMENU)); 
+#ifdef HAVE_WINDOWS_H
+                	uim.add(new UiSetting(SDLK_g, (format == GDI), "  Use Windows' GDI",   d));
+			if (format != GDI)
+			{
+#endif
+					
+             		        string ostat = "  Output is to ";
+                        	if (strdir[0] == '|') ostat += "pipe: " + displayName(strdir.substr(1), 40) + "  ";
+                        	else                  ostat += "directory: " + displayName(strdir, 40) + "  "; 
+                        	uim.add(new UiSeparator(d));
+                        	uim.add(new UiLabel  (ostat, d));
+                        	uim.add(new UiCommand(SDLK_r,  "  Output to diRectory  ", d));
+				uim.add(new UiCommand(SDLK_u,  "  Output to UNIX command... ",d, UIG_SUBMENU)); 
+#ifdef HAVE_WINDOWS_H
+			}
+#endif
                 }
 
                 d->centreContainer(uim);
@@ -584,6 +600,9 @@ UiEvent JoyceMatrix::settings(UiDrawer *d)
                 {
 			case SDLK_n:    format = PNG; break;
 			case SDLK_o:    format = PS; break;
+#ifdef HAVE_WINDOWS_H
+			case SDLK_g:    format = GDI; break;
+#endif
 			case SDLK_4:	m_w = 2997; m_h = 4208; break;
 			case SDLK_l:	m_w = 2997; m_h = 2104;	break;
 			case SDLK_5:	m_w = 2104; m_h = 2997; break;
@@ -595,16 +614,21 @@ UiEvent JoyceMatrix::settings(UiDrawer *d)
                         case SDLK_r:	m_y = MAX_H + 1;
 					checkEndForm();
 					if (format == PNG) 
-						strdir = findJoyceDir(FT_PNGS, false);
-					else	strdir = findJoyceDir(FT_PS,   false);
+						strdir = findPcwDir(FT_PNGS, false);
+					else	strdir = findPcwDir(FT_PS,   false);
 					break;
                         case SDLK_d:	enable(false); break;
                         case SDLK_c:	enable();      break;
                         case SDLK_u:	m_y = MAX_H + 1;
+					oldfn = m_llfilename;
+					olddr = m_lldriver;
 					checkEndForm();
                                         uie = getPipe(d);
                                         if (uie == UIE_QUIT) return uie;
-					strdir = m_filename;
+					strdir = "|";
+					strdir += m_llfilename;
+					m_llfilename = oldfn;
+					m_lldriver = olddr;
                                         break;
 
                         case SDLK_ESCAPE: return UIE_OK;
@@ -844,6 +868,12 @@ bool JoyceMatrix::startPage(void)
 	{
 		case PS:  return startPagePS();
 		case PNG: return startPagePNG();
+#ifdef HAVE_WINDOWS_H
+		case GDI: 
+			  if (m_drawing_page) return true;
+			  m_drawing_page = m_gdiPrint.startPage();
+			  return m_drawing_page;
+#endif
 	}
 	return false;
 }
@@ -855,6 +885,10 @@ bool JoyceMatrix::endPage(void)
 	{
 		case PS:  return endPagePS();
 		case PNG: return endPagePNG();
+#ifdef HAVE_WINDOWS_H
+		case GDI: m_drawing_page = false;
+			  return m_gdiPrint.endPage();
+#endif
 	}
 	return false;
 }
@@ -871,6 +905,9 @@ void JoyceMatrix::dot(float xf, float yf)
 	{
 		case PS:  dotPS(xf, yf);   break;
 		case PNG: dotPNG(xf, yf);  break;
+#ifdef HAVE_WINDOWS_H
+		case GDI: m_gdiPrint.putDot(xf, yf); break;
+#endif
 	}
 }
 
