@@ -1,8 +1,8 @@
 /************************************************************************
 
-    JOYCE v1.90 - Amstrad PCW emulator
+    JOYCE v2.1.0 - Amstrad PCW emulator
 
-    Copyright (C) 1996, 2001  John Elliott <jce@seasip.demon.co.uk>
+    Copyright (C) 1996, 2001-2  John Elliott <jce@seasip.demon.co.uk>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,9 +31,10 @@
 #include "UiColourEntry.hxx"
 
 
-JoycePcwTerm::JoycePcwTerm() : JoyceTerminal("pcw")
+JoycePcwTerm::JoycePcwTerm(PcwBeeper *b) : PcwTerminal("pcw")
 {
 	m_beepMode = BM_SOUND;
+	m_beeper   = b;
 	initSound();
 
 	m_beepFg.g = m_beepFg.b = 0x00;
@@ -49,7 +50,6 @@ JoycePcwTerm::JoycePcwTerm() : JoyceTerminal("pcw")
 
 JoycePcwTerm::~JoycePcwTerm()
 {
-	deinitSound();
 }
 
 //
@@ -93,11 +93,16 @@ static byte         gl_debugMode = false;
 		bytes += 8;			\
 		}
 
-
+/*
 #define PLINE_8 \
 		PLINE_BEGIN					\
                        data[yp++]=(v & 0x80 ? gl_whi:gl_blk);	\
 		PLINE_END(8)
+*/
+#define PLINE_8 \
+		PLINE_BEGIN					\
+                       data[yp++]=(v & 0x80 ? gl_pxw:gl_pxb);	\
+		PLINE_END(8)
 
 #define PLINE_16 \
 		PLINE_BEGIN					\
@@ -253,9 +258,6 @@ void JoycePcwTerm::enable(bool b)
 
 void JoycePcwTerm::reset(void)
 {	
-	m_ticks = 0;
-	m_wave.recpos = 0;
-	m_wave.soundpos = 0;
 	m_scrEnabled = 1;
 	m_scrCount = 0;
 	m_colourReg = 0;
@@ -289,7 +291,7 @@ void JoycePcwTerm::onGainFocus(void)
 {
 	m_visible = 1;
 	gl_shadowInvalid = 1;
-	JoyceTerminal::onGainFocus();
+	PcwTerminal::onGainFocus();
 	border(1);
 }
 
@@ -297,7 +299,7 @@ void JoycePcwTerm::onGainFocus(void)
 void JoycePcwTerm::onLoseFocus(void)
 {
         m_visible = 0;
-        JoyceTerminal::onLoseFocus();
+        PcwTerminal::onLoseFocus();
 }
 
 
@@ -377,7 +379,7 @@ void JoycePcwTerm::drawPcwScr(void)	/* Draw the PCW screen */
 		}
 		gl_shadowInvalid = 0;
 	}
-	r = gl_sys->m_termMenu.drawPopup();
+	r = gl_sys->m_termMenu->drawPopup();
 
 	if (r >  0) touched = 1;
 	if (r == 2) gl_shadowInvalid = 1;	
@@ -442,7 +444,7 @@ bool JoycePcwTerm::parseNode(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
 		if (!strcmp(s, "border")) m_beepMode = BM_BORDER;
 		if (!strcmp(s, "sound"))  m_beepMode = BM_SOUND;
 		if (!strcmp(s, "none"))   m_beepMode = BM_NONE;
-		free(s);
+		xmlFree(s);
 	}
 	if (!parseColour(doc, ns, cur, "beep", m_beepFg, m_beepBg)) return false;
         return parseColour(doc, ns, cur, "colour", m_screenFg, m_screenBg);
@@ -547,11 +549,11 @@ void JoycePcwTerm::onDebugSwitch(bool b)
 		gl_shadowInvalid = 1;
 		m_borderInvalid = 1;
 	}
-	JoyceTerminal::onDebugSwitch(b);
+	PcwTerminal::onDebugSwitch(b);
 }
 
 
-JoyceTerminal &JoycePcwTerm::operator << (unsigned char c)
+PcwTerminal &JoycePcwTerm::operator << (unsigned char c)
 {
 	// XXX to be done
 	return (*this);
@@ -679,3 +681,45 @@ UiEvent JoycePcwTerm::settings(UiDrawer *d)
 	return UIE_CONTINUE;
 }
 
+
+
+void JoycePcwTerm::tick(void)
+{
+	if (m_beeper) m_beeper->tick();
+}
+
+
+
+void JoycePcwTerm::beepOn(void)
+{
+	PcwTerminal::beepOn();
+	switch(m_beepMode)
+	{
+		case BM_BORDER:  m_borderInvalid = 1; border(1); break;
+		case BM_SOUND:   if (m_beeper) m_beeper->beepOn(); break;
+		default:	 break;
+	}
+}
+
+void JoycePcwTerm::beepOff(void)
+{
+	PcwTerminal::beepOff();
+	switch(m_beepMode)
+	{
+		case BM_BORDER:  m_borderInvalid = 1; border(1); break;
+		case BM_SOUND:   if (m_beeper) m_beeper->beepOff(); break;
+		default:	 break;
+	}
+}
+
+
+void JoycePcwTerm::deinitSound(void)
+{
+	if (m_beeper) m_beeper->deinitSound();
+}
+
+void JoycePcwTerm::initSound(void)
+{
+	if (m_beeper) m_beeper->initSound();
+}
+
