@@ -1,8 +1,8 @@
 /************************************************************************
 
-    JOYCE v1.90 - Amstrad PCW emulator
+    JOYCE v2.1.7 - Amstrad PCW emulator
 
-    Copyright (C) 1996, 2001  John Elliott <jce@seasip.demon.co.uk>
+    Copyright (C) 1996, 2001-5  John Elliott <jce@seasip.demon.co.uk>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -29,8 +29,8 @@
 #include "UiMenu.hxx"
 #include "UiTextInput.hxx"
 #include "UiScrollingMenu.hxx"
-#include "JoyceFileList.hxx"
-#include "JoyceFileChooser.hxx"
+#include "PcwFileList.hxx"
+#include "PcwFileChooser.hxx"
 
 static int addBootEntry(JoyceSystem *s, int id, const string title)
 {
@@ -76,12 +76,12 @@ static int migrateOldBoot(JoyceSystem *sys, FILE *fp)
 			t = strchr(s, '\n'); if (t) *t = 0;
 			t = strchr(s, '\r'); if (t) *t = 0;
 			sprintf(buf2, "boot%s.dsk", buf);
-			fpboot = openJoyceFile(FT_OLDBOOT, buf2, "rb");
+			fpboot = openPcwFile(FT_OLDBOOT, buf2, "rb");
 			if (!fpboot)
 			{
 				string err = "Cannot open ";
 				err += buf2;
-				uie = sys->m_termMenu.report(err, "  Skip this disc  ");
+				uie = sys->m_termMenu->report(err, "  Skip this disc  ");
 				if (uie == UIE_QUIT) return -1;
 				continue;	
 			}
@@ -90,7 +90,7 @@ static int migrateOldBoot(JoyceSystem *sys, FILE *fp)
 			doing += " (";
 			doing += s;
 			doing += ")";
-			sys->m_termMenu.popupRedraw(doing, 1);
+			sys->m_termMenu->popupRedraw(doing, 1);
 			/* Load the old DSK file into the TPA */
 			nb = 0;
 			while ((ch = fgetc(fpboot)) != EOF)
@@ -99,14 +99,14 @@ static int migrateOldBoot(JoyceSystem *sys, FILE *fp)
 			}
 			fclose(fpboot);
 			/* Write out the new DSK file */
-			fpboot = openJoyceFile(FT_BOOT, buf2, "wb");
+			fpboot = openPcwFile(FT_BOOT, buf2, "wb");
 			if (!fpboot)
 			{
 				string err = "Cannot create ";
 				err += buf2;
-				uie = sys->m_termMenu.report(err, "  Skip this disc  ");
+				uie = sys->m_termMenu->report(err, "  Skip this disc  ");
 				if (uie == UIE_QUIT) return -1;
-				sys->m_termMenu.popupOff();
+				sys->m_termMenu->popupOff();
 				continue;	
 			}
 			for (ch = 0; ch < nb; ch++)
@@ -121,16 +121,16 @@ static int migrateOldBoot(JoyceSystem *sys, FILE *fp)
 		}
 	}	
 	fclose(fp);
-	sys->m_termMenu.popupRedraw("Saving settings", 0);
+	sys->m_termMenu->popupRedraw("Saving settings", 0);
 	sys->storeBootList();
 	sys->saveBootList();
-	sys->m_termMenu.popupOff();
+	sys->m_termMenu->popupOff();
 	return count; 
 }
 
-static char *dsktypes[] = 
+static const char *dsktypes[] = 
 { 
-	NULL, "floppy", "dsk", "raw", "myz80"
+	NULL, "floppy", "dsk", "raw", "myz80", "cfi", "edsk", "ntwdm"
 };
 
 UiEvent getDskType(UiDrawer *d, int &fmt, bool create, bool &md3)
@@ -155,6 +155,11 @@ UiEvent getDskType(UiDrawer *d, int &fmt, bool create, bool &md3)
 		uim.add(new UiSetting(SDLK_2, (fmt == 2), "  CPCEMU/JOYCE .DSK  ", d));
 		uim.add(new UiSetting(SDLK_3, (fmt == 3), "  Raw disc image file  ", d));
 		uim.add(new UiSetting(SDLK_4, (fmt == 4), "  MYZ80 hard drive  ", d));
+		uim.add(new UiSetting(SDLK_5, (fmt == 5), "  .CFI - compressed raw  ", d));
+		uim.add(new UiSetting(SDLK_6, (fmt == 6), "  Extended .DSK   ", d));
+#ifdef HAVE_WINDOWS_H
+		uim.add(new UiSetting(SDLK_7, (fmt == 7), "  Alternative floppy  ", d));
+#endif
 		uim.add(new UiSeparator(d));
 		uim.add(new UiSetting(SDLK_n, !md3, "  Normal disc    ", d));
 		uim.add(new UiSetting(SDLK_m,  md3, "  MicroDesign program disc  ", d));
@@ -180,6 +185,8 @@ UiEvent getDskType(UiDrawer *d, int &fmt, bool create, bool &md3)
 			case SDLK_2: fmt = 2; break;
 			case SDLK_3: fmt = 3; break;
 			case SDLK_4: fmt = 4; break;
+			case SDLK_5: fmt = 5; break;
+			case SDLK_6: fmt = 6; break;
 			case SDLK_ESCAPE: 
 					md3 = omd3;
 					fmt = ofmt;
@@ -235,8 +242,10 @@ UiEvent requestDsk(UiDrawer *d, const string prompt, DSK_PDRIVER *dr,
 			case SDLK_c: inMenu = 0; break;
 			case SDLK_d: 
                                 {
-                                JoyceFileChooser f("  OK  ", d);
-                                f.initDir(filename);
+                                PcwFileChooser f("  OK  ", d);
+                                if (filename == A_DRIVE || filename == B_DRIVE)
+                                        f.initDir(findPcwDir(FT_BOOT, false));
+                                else    f.initDir(filename);
                                 uie = f.eventLoop();
                                 if (uie == UIE_QUIT) return uie;
                                 if (uie == UIE_OK) filename = f.getChoice();
@@ -410,7 +419,7 @@ static dsk_err_t dskTrans(JoyceSystem *sys, DSK_PDRIVER indr, DSK_PDRIVER outdr,
                 cyl +1, dg.dg_cylinders,
                 head+1, dg.dg_heads,
                 sec+dg.dg_secbase, dg.dg_sectors + dg.dg_secbase - 1); 
-				sys->m_termMenu.popupRedraw(progress, 1);
+				sys->m_termMenu->popupRedraw(progress, 1);
                         
                                 op = "Reading"; 
                                 e = dsk_pread(indr, &dg, buf, cyl,head,sec + dg.dg_secbase);
@@ -426,7 +435,7 @@ static dsk_err_t dskTrans(JoyceSystem *sys, DSK_PDRIVER indr, DSK_PDRIVER outdr,
                 }
                 if (e) break;
         }
-	sys->m_termMenu.popupOff();
+	sys->m_termMenu->popupOff();
 	free(buf);
 	return e;       
 }
@@ -443,7 +452,7 @@ UiEvent addBoot(JoyceSystem *sys, int *successes)
 	int ispcw;
 	string why = "Not a PCW boot disc";
 	string newName, shortName;
-	UiDrawer *d = &sys->m_termMenu;
+	UiDrawer *d = sys->m_termMenu;
 
 	e = requestDsk(d, "  Add boot disc  ", &drfrom, &dg, false, md3);
 	if (e != UIE_OK) return e;
@@ -476,17 +485,17 @@ UiEvent addBoot(JoyceSystem *sys, int *successes)
 		dsk_close(&drfrom);
 		e = showDskErr(err, why, d);
 		if (e == UIE_QUIT) return e;
-		return sys->m_termMenu.yesno("Add another boot disc?");
+		return sys->m_termMenu->yesno("Add another boot disc?");
 	}
 	shortName = newBootFilename(sys, id);
-	newName = findJoyceFile(FT_BOOT, shortName, "wb");
+	newName = findPcwFile(FT_BOOT, shortName, "wb");
 
 	e = getBootName(d, shortName);	
 	if (e != UIE_OK)
 	{
 		dsk_close(&drfrom);
 		if (e == UIE_QUIT) return e;
-		return sys->m_termMenu.yesno("Add another boot disc?");
+		return sys->m_termMenu->yesno("Add another boot disc?");
 	}
 	err = dsk_creat(&drto, newName.c_str(), "dsk", NULL);
 	if (!err) err = dskTrans(sys, drfrom, drto, dg, md3);		
@@ -504,7 +513,7 @@ UiEvent addBoot(JoyceSystem *sys, int *successes)
 		addBootEntry(sys, id, shortName);
 	}
 	// Now ask whether to do another.
-	return sys->m_termMenu.yesno("Add another boot disc?");
+	return sys->m_termMenu->yesno("Add another boot disc?");
 }
 
 
@@ -517,7 +526,7 @@ UiEvent readDsk(JoyceSystem *sys)
 	int ispcw;
 	string why = "Not a PCW disc";
 	string newName, shortName;
-	UiDrawer *d = &sys->m_termMenu;
+	UiDrawer *d = sys->m_termMenu;
 
 	e = requestDsk(d, "  Read in a disc  ", &drfrom, &dg, false, md3);
 	if (e != UIE_OK) return e;
@@ -526,7 +535,7 @@ UiEvent readDsk(JoyceSystem *sys)
 
 	e = getDiscName(d, shortName);
 	if (e != UIE_OK) return e;
-	newName   = findJoyceFile(FT_DISC, shortName, "wb");
+	newName   = findPcwFile(FT_DISC, shortName, "wb");
 	err = dsk_creat(&drto, newName.c_str(), "dsk", NULL);
 	if (!err) err = dskTrans(sys, drfrom, drto, dg, md3);		
 
@@ -549,10 +558,10 @@ UiEvent writeDsk(JoyceSystem *sys)
 	int ispcw;
 	string why = "Failed to open DSK file";
 	string oldName;
-	UiDrawer *d = &sys->m_termMenu;
+	UiDrawer *d = sys->m_termMenu;
 
-	JoyceFileChooser f("  OK  ", d);
-	f.initDir(findJoyceDir(FT_DISC, false));
+	PcwFileChooser f("  OK  ", d);
+	f.initDir(findPcwDir(FT_DISC, false));
 	e = f.eventLoop();
         if (e != UIE_OK) return e;
   
@@ -589,34 +598,34 @@ UiEvent writeDsk(JoyceSystem *sys)
 /* Returns: -1 to quit 
  * 0 if no discs read in
  * >0 if discs were read in */
-int autoSetup(JoyceSystem *sys)
+int JoyceSystem::autoSetup()
 {
 	UiEvent e;
 	int successes = 0;
 
 	/* 1. See if there's an existing boot.cfg */
-	FILE *fp = openJoyceFile(FT_OLDBOOT, "boot.cfg", "r");
+	FILE *fp = openPcwFile(FT_OLDBOOT, "boot.cfg", "r");
 	if (fp)
 	{
-		e = sys->m_termMenu.yesno(
+		e = m_termMenu->yesno(
 			"Copy boot files from JOYCE 1.3x system?");
-		if (e == UIE_OK) return migrateOldBoot(sys, fp);
+		if (e == UIE_OK) return migrateOldBoot(this, fp);
 		fclose(fp);
 	}
 	/* 2. Add new... */
 	do
 	{
-		e = addBoot(sys, &successes);
+		e = addBoot(this, &successes);
 		if (e == UIE_QUIT) return -1;
 	} while (e != UIE_CANCEL);
 
-	sys->storeBootList();
-	sys->saveBootList();
+	storeBootList();
+	saveBootList();
 	return successes;
 }
 
 
-UiEvent discManager(JoyceSystem *sys, UiDrawer *d)
+UiEvent JoyceSystem::discManager(UiDrawer *d)
 {
 	UiEvent uie;
 	int inMenu = 1;
@@ -649,32 +658,32 @@ UiEvent discManager(JoyceSystem *sys, UiDrawer *d)
 		sel = uim.getSelected();
 		switch(uim.getKey(sel))
 		{
-			case SDLK_a: uie = addBoot(sys, &successes);
+			case SDLK_a: uie = addBoot(this, &successes);
 				     if (uie == UIE_QUIT) return uie;
 				     break;
-			case SDLK_r: uie = sys->chooseBootID(d, "  Choose boot disc to rename  ", &bootid);
+			case SDLK_r: uie = chooseBootID(d, "  Choose boot disc to rename  ", &bootid);
 				     if (uie == UIE_QUIT) return uie;
-				     bootno = sys->lookupId(bootid); 
+				     bootno = lookupId(bootid); 
 				     if (bootno < 0) break;
-				     uie = getBootName(d, name, sys->m_bootList[bootno].m_title);
+				     uie = getBootName(d, name, m_bootList[bootno].m_title);
 				     if (uie == UIE_QUIT) return uie;
-			             if (uie == UIE_OK) sys->m_bootList[bootno].m_title = name;
+			             if (uie == UIE_OK) m_bootList[bootno].m_title = name;
 				     break;
-			case SDLK_d: uie = sys->chooseBootID(d, "  Choose boot disc to delete  ", &bootid);
+			case SDLK_d: uie = chooseBootID(d, "  Choose boot disc to delete  ", &bootid);
 				     if (uie == UIE_QUIT) return uie;
-				     sys->deleteBootEntry(bootid);
+				     deleteBootEntry(bootid);
 				     break;
-			case SDLK_c: uie = readDsk(sys);
+			case SDLK_c: uie = readDsk(this);
 				     if (uie == UIE_QUIT) return uie;
 				     break;
-			case SDLK_w: uie = writeDsk(sys);
+			case SDLK_w: uie = writeDsk(this);
 				     if (uie == UIE_QUIT) return uie;
 				     break;
 			case SDLK_ESCAPE: 
-				sys->m_termMenu.popupRedraw("Saving settings", 0);
-				sys->storeBootList();
-				sys->saveBootList();
-				sys->m_termMenu.popupOff();
+				m_termMenu->popupRedraw("Saving settings", 0);
+				storeBootList();
+				saveBootList();
+				m_termMenu->popupOff();
 				return UIE_OK;
 			default:     break;
 		}
