@@ -1,9 +1,9 @@
 
 /************************************************************************
 
-    JOYCE v1.90 - Amstrad PCW emulator
+    JOYCE v2.1.0 - Amstrad PCW emulator
 
-    Copyright (C) 1996, 2001  John Elliott <jce@seasip.demon.co.uk>
+    Copyright (C) 1996, 2001-2  John Elliott <jce@seasip.demon.co.uk>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -30,8 +30,8 @@
 #include "UiMenu.hxx"
 #include "UiTextInput.hxx"
 #include "UiScrollingMenu.hxx"
-#include "JoyceFileList.hxx"
-#include "JoyceFileChooser.hxx"
+#include "PcwFileList.hxx"
+#include "PcwFileChooser.hxx"
 
 //
 // Minimal bootstrap ROM to run a boot sector that's already loaded at
@@ -42,28 +42,75 @@ static const byte initBoot[] = { 0xF3, 0x31, 0x00, 0xF0, 0xC3, 0x10, 0xF0 };
 //
 // Namespace for the hardware definitions
 //
-#define XMLNS ((xmlChar *)"http://www.seasip.demon.co.uk/joycecfg.dtd")
+#define XMLNS             "http://www.seasip.demon.co.uk/joycecfg.dtd"
 #define HW_FILE           "joycehw.xml" 
-
 //
 // Namespace for the boot disc list
 //
 #define CFGNS ((xmlChar *)"http://www.seasip.demon.co.uk/joycebt.dtd")
 #define BOOT_FILE         "joycebt.xml" 
+
 //
 // This class is supposed to encapsulate the entire PCW system. 
 //
+JoyceSystem::JoyceSystem(JoyceArgs *args) : PcwSystem(args, HW_FILE, XMLNS),
+		        m_keyb(this), 
+			m_mouse(this),
+                        m_daisy(this),
+                        m_cps(this)
+{
+        m_cpu = new JoyceZ80(this);
+	m_asic = new JoyceAsic(this);
+	m_mem  = new JoyceMemory();
+        if (m_bootfile == "")
+                m_bootfile = findPcwFile(FT_BOOT, BOOT_SYS, "rb");
+// Need to create m_termMenu before m_fdc
+	m_termMenu       = new JoyceMenuTermPcw(this);
+	m_fdc = new JoyceFDC(this);
+
+	m_activeKeyboard = &m_keyb;	
+	m_activeMouse    = &m_mouse;
+
+	m_bootcfg = xmlNewDoc((xmlChar *)"1.0");
+	if (m_bootcfg)
+	{
+	        xmlNsPtr ns;
+       		xmlNodePtr root;
+
+	        root = xmlNewNode(NULL, (xmlChar *)"bootlist");
+       		xmlDocSetRootElement(m_bootcfg, root);
+
+        	ns = xmlNewNs(root, CFGNS, NULL);
+	}
+	m_defaultTerminal = new JoycePcwTerm(&m_beeper);
+	m_activePrinter  = &m_cen;
+	addDevice(m_fdc);
+	addDevice(m_asic);
+	addDevice(m_mem);
+	addDevice(m_defaultTerminal);
+	addDevice(m_termMenu);
+	addDevice(&m_joystick);
+	addDevice(&m_keyb);
+	addDevice(&m_mouse);
+	addDevice(&m_locoLink);
+        addDevice(&m_matrix);
+        addDevice(&m_daisy);
+        addDevice(&m_cen);
+        addDevice(&m_cps);
+}
 
 
-
+JoyceSystem::~JoyceSystem()
+{
+	delete m_defaultTerminal;
+        if (m_bootcfg)  xmlFreeDoc(m_bootcfg);
+}
+#if 0
 JoyceSystem::JoyceSystem(JoyceArgs *args) : 
-			     m_cpu(this), 
 			     m_asic(this),
 			     m_fdc(this),
 			     m_daisy(this),
 			     m_cps(this),
-			     m_keyb(this), 
-			     m_mouse(this),
 			     m_termVga(this), 
 			     m_termLogo(this), 
                              m_termMenu(this), 
@@ -71,12 +118,10 @@ JoyceSystem::JoyceSystem(JoyceArgs *args) :
 {
 	XLOG("JoyceSystem::JoyceSystem()");
 
+        m_cpu = new JoyceZ80(this); 
 	m_bootfile = args->m_bootRom;
 	m_args     = args;
 
-	m_activePrinter  = &m_cen;
-	m_activeKeyboard = &m_keyb;
-	m_activeMouse    = &m_mouse;
 	m_activeTerminal = NULL;
 	m_video          = NULL;
 	m_bootRom        = NULL;
@@ -100,19 +145,8 @@ JoyceSystem::JoyceSystem(JoyceArgs *args) :
 
         	ns = xmlNewNs(root, XMLNS, NULL);
 	}
-	m_bootcfg = xmlNewDoc((xmlChar *)"1.0");
-	if (m_bootcfg)
-	{
-	        xmlNsPtr ns;
-       		xmlNodePtr root;
 
-	        root = xmlNewNode(NULL, (xmlChar *)"bootlist");
-       		xmlDocSetRootElement(m_bootcfg, root);
-
-        	ns = xmlNewNs(root, CFGNS, NULL);
-	}
 	// Add all devices to generic list
-	addDevice(&m_asic);
 	addDevice(&m_mem);
 	addDevice(&m_fdc);
 	addDevice(&m_hdc);
@@ -120,8 +154,6 @@ JoyceSystem::JoyceSystem(JoyceArgs *args) :
 	addDevice(&m_daisy);
 	addDevice(&m_cen);
 	addDevice(&m_cps);
-	addDevice(&m_keyb);
-	addDevice(&m_mouse);
 	addDevice(&m_termPcw);
 	addDevice(&m_termLogo);
 	addDevice(&m_termVga);
@@ -135,6 +167,7 @@ JoyceSystem::~JoyceSystem()
         if (m_settings) xmlFreeDoc(m_settings);
         if (m_bootcfg)  xmlFreeDoc(m_bootcfg);
 	if (m_bootRom)  delete m_bootRom;
+	delete m_cpu;
 }
 
 
@@ -165,7 +198,7 @@ bool JoyceSystem::loadHardware()
         xmlNsPtr ns;
         xmlNodePtr cur;
 
-        filename = findJoyceFile(FT_NORMAL, HW_FILE, "r");
+        filename = findPcwFile(FT_NORMAL, HW_FILE, "r");
 //
 // Do a lot of sanity checks on the hardware XML.
 //
@@ -202,23 +235,17 @@ bool JoyceSystem::loadHardware()
 }
 
 
-
+#endif
 bool JoyceSystem::parseHardware()
 {
-	xmlNodePtr root = xmlDocGetRootElement(m_settings);
-	xmlNsPtr   ns   = xmlSearchNsByHref(m_settings, root, XMLNS);
-	bool okay = true;
+	bool okay = PcwSystem::parseHardware();
 
-        for (int n = 0; n < m_devCount; n++) 
-	{
-		okay = okay && m_devices[n]->parse(m_settings, ns, root);
-	}
 	// Allow the FDC to alter the PCW model type
-	setModel(m_fdc.adaptModel(getModel()));
+	setModel(((JoyceFDC *)m_fdc)->adaptModel(getModel()));
 
 	return okay;
 }
-
+#if 0
 
 bool JoyceSystem::storeHardware()
 {
@@ -240,7 +267,7 @@ bool JoyceSystem::saveHardware()
 
 	if (!m_settings) return false;
 
-        filename = findJoyceFile(FT_NORMAL,HW_FILE,"w"); 
+        filename = findPcwFile(FT_NORMAL,HW_FILE,"w"); 
 
         if (xmlSaveFile(filename.c_str(), m_settings) < 0) return 0;
         return 1;
@@ -274,45 +301,39 @@ bool JoyceSystem::initVideo(void)
 	}
 	m_termPcw.  setSysVideo(m_video);
 	m_termLogo. setSysVideo(m_video);
-        m_termMenu. setSysVideo(m_video);
+        m_termMenu-> setSysVideo(m_video);
         m_termVga.  setSysVideo(m_video);
 	m_termDebug.setSysVideo(m_video);
 
 	// General system reset
 	reset();
-        m_termMenu. setSysVideo(m_video);
+        m_termMenu-> setSysVideo(m_video);
 
 	return true;
 }
+#endif
 
 
 bool JoyceSystem::deinitVideo(void)
 {
+	// [v2.1.5] Similarly, shut down joysticks
+	m_keyb.reset();
+	m_joystick.reset();
 	// [v1.9.3] Shut down sound before releasing SDL
-	m_termPcw.deinitSound();
-	if (m_video) delete m_video;
-	return true;
+        ((JoycePcwTerm *)m_defaultTerminal)->deinitSound();
+	PcwSystem::deinitVideo();
 }
 
-
 ///////////////////////////////////////////////////////////////////////////////
 //
 // General reset
 //
 void JoyceSystem::reset(void)
 {
-	m_resetPending = false;
-	for (int n = 0; n < m_devCount; n++) m_devices[n]->reset();
-	// Load the initial boot ROM
-	loadBoot(m_bootfile);
-	// Switch to PCW terminal, that's what the boot ROM will be using
-	selectTerminal(&m_termPcw);	
-}
-
-void JoyceSystem::setResetFlag(bool b)
-{
-	m_resetPending = b;
+	PcwSystem::reset();
+	loadBoot(PCWRAM, m_bootfile);
 }
+#if 0
 
 
 //////////////////////////////////////////////////////////////////////////////
@@ -354,8 +375,8 @@ UiEvent  JoyceSystem::settings(UiDrawer *d, int x, int y)
 			if (uim.getKey(n) == SDLK_s) 
 			{
 				storeHardware();
-				if (saveHardware()) uie = m_termMenu.report("Settings were saved", "  OK");
-				else		    uie = m_termMenu.report("Failed to save settings","  Cancel");
+				if (saveHardware()) uie = m_termMenu->report("Settings were saved", "  OK");
+				else		    uie = m_termMenu->report("Failed to save settings","  Cancel");
  
 				if (uie == UIE_QUIT) return uie;
 			}
@@ -413,7 +434,7 @@ static void zputs(word W, const char *s)
                 ++n;
         }
 }
-
+#endif
 
 bool JoyceSystem::loadBootList()
 {
@@ -422,7 +443,7 @@ bool JoyceSystem::loadBootList()
         xmlNsPtr ns;
         xmlNodePtr cur;
 
-        filename = findJoyceFile(FT_BOOT, BOOT_FILE, "r");
+        filename = findPcwFile(FT_BOOT, BOOT_FILE, "r");
 //
 // Do a lot of sanity checks on the XML.
 //
@@ -478,15 +499,15 @@ bool JoyceSystem::parseBootList()
 		        if (!s || s[0] < '0' || s[0] > '9')
 		        {
                 		joyce_dprintf("<bootdisc> must have an id= field\n");
-               			if (s) free(s);
+               			if (s) xmlFree(s);
 				continue;
 			} 
 			e.m_id = atoi((char *)s);
-      			free(s); 
+      			xmlFree(s); 
 			s = xmlNodeListGetString(m_bootcfg, cur->xmlChildrenNode, 1);
 			if (!s) continue;
 			e.m_title = (char *)s;
-			free(s);
+			xmlFree(s);
 			m_bootList.push_back(e);				
                 }
                 cur = cur->next;
@@ -530,11 +551,11 @@ bool JoyceSystem::storeBootEntry(int n)
 		        if (!s || s[0] < '0' || s[0] > '9')
 		        {
                 		joyce_dprintf("<bootdisc> must have an id= field\n");
-               			if (s) free(s);
+               			if (s) xmlFree(s);
 				continue;
 			} 
 			if (e.m_id != atoi((char *)s)) { free(s); continue; }
-      			free(s); 
+      			xmlFree(s); 
 	
 			++found;
 			xmlReplaceNode(cur, newNode);
@@ -556,7 +577,7 @@ bool JoyceSystem::saveBootList()
 
 	if (!m_bootcfg) return false;
 
-        filename = findJoyceFile(FT_BOOT, BOOT_FILE, "w"); 
+        filename = findPcwFile(FT_BOOT, BOOT_FILE, "w"); 
 
         if (xmlSaveFile(filename.c_str(), m_bootcfg) < 0) return 0;
         return 1;
@@ -564,12 +585,12 @@ bool JoyceSystem::saveBootList()
 
 UiEvent JoyceSystem::bootHelp()
 {
-	UiDrawer *d = (UiDrawer *)&m_termMenu;
+	UiDrawer *d = m_termMenu;
 	UiMenu uim(d);
 	
-	uim.add(new UiLabel("This screen lists up to 10 boot discs (start-of-day discs)", d));
+	uim.add(new UiLabel("This screen lists up to 9 boot discs (start-of-day discs)", d));
 	uim.add(new UiLabel("that you have read into JOYCE. If you have read in more ", d));
-	uim.add(new UiLabel("than 10, the others will be listed if you press the [F3] ", d));
+	uim.add(new UiLabel("than 9, the others will be listed if you press the [F3] ", d));
 	uim.add(new UiLabel("key. ", d));
 	uim.add(new UiLabel("  To read in more boot discs, press [F9] key, then [F2]", d));
 	uim.add(new UiLabel("and choose 'Disc management' from the menu that appears.", d));
@@ -585,7 +606,7 @@ void JoyceSystem::getBootID(Z80 *R)
 	unsigned n;
 	UiEvent e;
 	string bootfile;
-	char *type = NULL;
+	const char *type = NULL;
 
 	// New in v1.5.0+: Extra boot image functions are overloaded in by
 	// checking B == 255. 
@@ -593,15 +614,15 @@ void JoyceSystem::getBootID(Z80 *R)
 	{
 		// Alternative boot 
 		case 0:  m_keyb.clearKeys();
-			 e = chooseBootDisc((UiDrawer *)&m_termMenu, bootfile, &type);
+			 e = chooseBootDisc(m_termMenu, bootfile, &type);
 			 if (e == UIE_QUIT) goodbye(99);
 			 if (e != UIE_OK) { R->a = 1; return; }
 			// Try to boot alternative image
 			 if (fdcBoot(bootfile, type))
                          {
 				memcpy(PCWRAM, initBoot, sizeof(initBoot));
-				m_cpu.pc = 0x0000;
-				m_cpu.iff1 = m_cpu.iff2 = 0;
+				m_cpu->pc = 0x0000;
+				m_cpu->iff1 = m_cpu->iff2 = 0;
 				return;
                          }
 			 memset(PCWRAM + 0xFFF0, 0, 16);
@@ -625,7 +646,7 @@ void JoyceSystem::getBootID(Z80 *R)
 	}
 }
 
-UiEvent JoyceSystem::chooseBootDisc(UiDrawer *d, string &bootfile, char **type)
+UiEvent JoyceSystem::chooseBootDisc(UiDrawer *d, string &bootfile, const char **type)
 {
 	UiMenu uim(d);
 	UiEvent uie;
@@ -672,7 +693,7 @@ UiEvent JoyceSystem::chooseBootDisc(UiDrawer *d, string &bootfile, char **type)
 //		f.initDir(filename);
 //              uie = f.eventLoop();
 
-		uie = m_fdc.requestDsk(d, bootfile, type);
+		uie = m_fdc->requestDsk(d, bootfile, type);
                 if (uie != UIE_OK) return uie;
 //                bootfile = f.getChoice();
 		return UIE_OK;
@@ -743,13 +764,13 @@ UiEvent JoyceSystem::chooseBootID(UiDrawer *d, const string prompt, int *bootid)
 
 
 
-int JoyceSystem::fdcBoot(const string filename, char *type)
+int JoyceSystem::fdcBoot(const string filename, const char *type)
 {
 	bool okay;
 	byte cksum;
 	int n, warn = 0;
 
-	okay = m_fdc.loadBootSector(filename, type, PCWRAM + 0xF000);
+	okay = m_fdc->loadBootSector(filename, type, PCWRAM + 0xF000);
 	if (!okay) return 0;
 
 	cksum = 0;
@@ -768,28 +789,28 @@ int JoyceSystem::fdcBoot(const string filename, char *type)
 			if (m_model == PCW10)    { warn = 1; setModel(PCW9000P); }
 			break;
 		case 0x02:
-			m_termMenu.popup("Can't boot from LocoScript install disc", 4, 1);
+			m_termMenu->popup("Can't boot from LocoScript install disc", 4, 1);
 			return 0;
 		case 0x03:
-			m_termMenu.popup("Can't boot from Spectrum +3 disc", 4, 1);
+			m_termMenu->popup("Can't boot from Spectrum +3 disc", 4, 1);
 			return 0;
 		default:
-			m_termMenu.popup("Unknown type of boot disc", 4, 1);
+			m_termMenu->popup("Unknown type of boot disc", 4, 1);
 			return 0;
 			
 		}
 	if (warn) switch(m_model)
 	{
-		case PCW8000:  m_termMenu.popup("PCW type changed to 8256/8512", 6, 1); break;
-		case PCW9000:  m_termMenu.popup("PCW type changed to 9512", 6, 1); break;
-		case PCW9000P: m_termMenu.popup("PCW type changed to 9512+", 6, 1); break;
-		case PCW10:    m_termMenu.popup("PCW type changed to 9256/10", 6, 1); break;
+		case PCW8000:  m_termMenu->popup("PCW type changed to 8256/8512", 6, 1); break;
+		case PCW9000:  m_termMenu->popup("PCW type changed to 9512", 6, 1); break;
+		case PCW9000P: m_termMenu->popup("PCW type changed to 9512+", 6, 1); break;
+		case PCW10:    m_termMenu->popup("PCW type changed to 9256/10", 6, 1); break;
 	}
         memset(PCWRAM, 0, 0xF000);
 	// [v1.9.2] Insert B drive disc if specified
 	if (m_args->m_bDrive != "")
 	{
-		m_fdc.insertB(m_args->m_bDrive);
+		m_fdc->insertB(m_args->m_bDrive);
 		m_args->m_bDrive = "";
 	}
         return 1;
@@ -809,7 +830,7 @@ string JoyceSystem::idToFilename(int bootid)
         bootfile = "boot";
         bootfile += nbr;
         bootfile += ".dsk";
-        bootfile = findJoyceFile(FT_BOOT, bootfile, "rb");
+        bootfile = findPcwFile(FT_BOOT, bootfile, "rb");
 	return bootfile;
 }
 
@@ -825,9 +846,9 @@ bool JoyceSystem::deleteBootEntry(int bootid)
 {
 	int n = lookupId(bootid);
 	if (n < 0) return false;
-	
+
 	// Remove disc from the boot list	
-	m_bootList.erase(&m_bootList[n]);
+	m_bootList.erase(m_bootList.begin() + n);
 	// Remove it from the XML	
 	xmlNodePtr cur  = xmlDocGetRootElement(m_bootcfg);
 	xmlNodePtr next;
@@ -844,14 +865,15 @@ bool JoyceSystem::deleteBootEntry(int bootid)
 		s = xmlGetProp(cur, (xmlChar *)"id");
 		if (!s) continue;
 
-		if (bootid != atoi((char *)s)) { free(s); continue; }
-		free(s);
+		if (bootid != atoi((char *)s)) { xmlFree(s); continue; }
+		xmlFree(s);
                 xmlUnlinkNode(cur);
                 xmlFreeNode(cur);
 	}
 	return true;
 }
 
+#if 0
 
 // Load the initial boot ROM (BOOTFILE.EMJ).
 bool JoyceSystem::loadBoot(string bootfile)
@@ -870,7 +892,7 @@ bool JoyceSystem::loadBoot(string bootfile)
 
         if ((fp = fopen(bootfile.c_str(),"rb")) == NULL)
         {
-                m_termMenu.report("Failed to open the boot ROM", "  Exit  ");
+                m_termMenu->report("Failed to open the boot ROM", "  Exit  ");
                 deinitVideo();
                 fprintf(stderr,"Open of %s failed!  \r\n", bootfile.c_str());
                 diewith("Failed to open boot file\r\n",98);
@@ -916,36 +938,30 @@ void JoyceSystem::setDebug(bool b)
 	else   m_termDebug.onLoseFocus();
 }
 
-
+#endif
 ///////////////////////////////////////////////////////////////////////////////
 //
 // System init
 //
 bool JoyceSystem::sysInit(void)
 {
-	int memsiz;
-
+        loadBootList();
+        parseBootList();
 	loadHardware();
 	parseHardware();
-	loadBootList();
-	parseBootList();
-
-	memsiz = m_mem.alloc(m_args);
-	if (!memsiz)
-	{
-		alert("Not enough memory to run JOYCE.");
-		return false;
-	}
-	printf("JOYCE will emulate a PCW 8%d (or 9%d)\n", 
-			memsiz, memsiz);
+        int memsiz = m_mem->alloc(m_args);
+        if (!memsiz)
+        {
+                alert("Not enough memory to run JOYCE.");
+                return false;
+        }
+        printf("JOYCE will emulate a PCW 8%d (or 9%d)\n", memsiz, memsiz);
 	fflush(stdout);
-	if (!initVideo())
-	{
-                alert("Sorry, could not select 800x600 graphics mode.\n"
-                      "JOYCE needs at least 800x600 8-bit graphics.");
-		return false;
-	}
-	setDebug(m_args->m_debug);
+
+	if (!PcwSystem::sysInit()) return false;
+
+	loadBoot(PCWRAM, m_bootfile);
+
 // All "outside functions" are regulated in JOYCE by a
 // simulated 900Hz clock. For a 4MHz CPU, this is every
 // 4e6/9e2 = 4444 cycles, or about 500 instructions. 
@@ -954,12 +970,9 @@ bool JoyceSystem::sysInit(void)
 // using 4441 rather than 4444 because it's prime, and so we're 
 // less likely to get problems caused by interrupts synchronising 
 // exactly with a tight loop.
-	m_cpu.m_cycles = 4441;
-	extern_stop = 0;	// XXX global
-	m_cpu.reset();
-	// Load the boot ROM.
-	m_mem.reset();
-	loadBoot(m_bootfile);
+//
+	m_cpu->setCycles(4441);
+	m_cpu->reset();
 	// If there is no BOOT.CFG, ask about adding some boot discs.
 	// If we've been explicitly told to boot from a disc, then
 	// it doesn't matter if any were set up or not.
@@ -967,7 +980,7 @@ bool JoyceSystem::sysInit(void)
 	{
 		int nc;
 		UiEvent uie;
-		UiDrawer *d = &m_termMenu;
+		UiDrawer *d = m_termMenu;
 		UiMenu uim(d);
 
 		uim.add(new UiLabel("  No JOYCE boot discs have been set up.  ", d));
@@ -983,10 +996,10 @@ bool JoyceSystem::sysInit(void)
 		{
 			case SDLK_q: return false;
 			case SDLK_s:
-				nc = autoSetup(this);
+				nc = autoSetup();
 				if (!nc)
 				{
-					m_termMenu.report("No discs were converted",
+					m_termMenu->report("No discs were converted",
                                                         "  Continue  ");
 				}
 				if (nc < 0) 
@@ -1014,8 +1027,8 @@ bool JoyceSystem::sysInit(void)
 // The Z80 always starts at 0000h. Put a bootstrap there and a jump to it
 // at the bottom of memory.
 				memcpy(PCWRAM, initBoot, sizeof(initBoot));
-				m_cpu.pc = 0x0000;
-				m_cpu.iff1 = m_cpu.iff2 = 0;
+				m_cpu->pc = 0x0000;
+				m_cpu->iff1 = m_cpu->iff2 = 0;
 				return true;
 			}
 			joyce_dprintf("Could not auto-boot disc %d\n", id);
@@ -1023,8 +1036,8 @@ bool JoyceSystem::sysInit(void)
 		else if (fdcBoot(m_args->m_aDrive))
 		{
 			memcpy(PCWRAM, initBoot, sizeof(initBoot));
-			m_cpu.pc = 0x0000;
-			m_cpu.iff1 = m_cpu.iff2 = 0;
+			m_cpu->pc = 0x0000;
+			m_cpu->iff1 = m_cpu->iff2 = 0;
 			return true;
 		}
 	}
