@@ -1,6 +1,5 @@
 
 #include "Joyce.hxx"
-#include "Joycevga.hxx"
 #include "keyboard.hxx"
 #include <assert.h>
 
@@ -14,8 +13,6 @@
 
 #define false 0
 
-void gsx_cmd(word pb) { assert(false); }
-
 ////////////////////////////////////////////////////////////////////////
 // Raw screen access
 //
